import matplotlib.backend_bases
import matplotlib.axes
class class_Figure:
    def function_set_canvas(self):
        l_canvas = matplotlib.backend_bases.FigureCanvasBase()
    def function_add_subplot(self):
        returns = matplotlib.axes.Axes()



# AUTO-GENERATED FILE -- DO NOT EDIT

""" Common string manipulations, optimized for speed.

Always use "import string" rather than referencing
this module directly. """

__package__ = None

def atof(s):
  """ atof(s) -> float
  
  Return the floating point number represented by the string s. """
  return 1.0

def atoi(s, base=None):
  """ atoi(s [,base]) -> int
  
  Return the integer represented by the string s in the given
  base, which defaults to 10.  The string s must consist of one
  or more digits, possibly preceded by a sign.  If base is 0, it
  is chosen from the leading characters of s, 0 for octal, 0x or
  0X for hexadecimal.  If base is 16, a preceding 0x or 0X is
  accepted. """
  return 1

def atol(s, base=None):
  """ atol(s [,base]) -> long
  
  Return the long integer represented by the string s in the
  given base, which defaults to 10.  The string s must consist
  of one or more digits, possibly preceded by a sign.  If base
  is 0, it is chosen from the leading characters of s, 0 for
  octal, 0x or 0X for hexadecimal.  If base is 16, a preceding
  0x or 0X is accepted.  A trailing L or l is not accepted,
  unless base is 0. """
  return None

def capitalize(s):
  """ capitalize(s) -> string
  
  Return a copy of the string s with only its first character
  capitalized. """
  return ""

def count(s, sub, start=None, end=None):
  """ count(s, sub[, start[, end]]) -> int
  
  Return the number of occurrences of substring sub in string
  s[start:end].  Optional arguments start and end are
  interpreted as in slice notation. """
  return 1

def expandtabs(string, tabsize=None):
  """ expandtabs(string, [tabsize]) -> string
  
  Expand tabs in a string, i.e. replace them by one or more spaces,
  depending on the current column and the given tab size (default 8).
  The column number is reset to zero after each newline occurring in the
  string.  This doesn't understand other non-printing characters. """
  return ""

def find(s, sub, start=None, end=None):
  """ find(s, sub [,start [,end]]) -> in
  
  Return the lowest index in s where substring sub is found,
  such that sub is contained within s[start,end].  Optional
  arguments start and end are interpreted as in slice notation.
  
  Return -1 on failure. """
  return None

def join(list, sep=None):
  """ join(list [,sep]) -> string
  joinfields(list [,sep]) -> string
  
  Return a string composed of the words in list, with
  intervening occurrences of sep.  Sep defaults to a single
  space.
  
  (join and joinfields are synonymous) """
  return ""

def joinfields(list, sep=None):
  """ join(list [,sep]) -> string
  joinfields(list [,sep]) -> string
  
  Return a string composed of the words in list, with
  intervening occurrences of sep.  Sep defaults to a single
  space.
  
  (join and joinfields are synonymous) """
  return ""

def lower(s):
  """ lower(s) -> string
  
  Return a copy of the string s converted to lowercase. """
  return ""

lowercase = 'abcdefghijklmnopqrstuvwxyz'

def lstrip(s):
  """ lstrip(s) -> string
  
  Return a copy of the string s with leading whitespace removed. """
  return ""

def maketrans(frm, to):
  """ maketrans(frm, to) -> string
  
  Return a translation table (a string of 256 bytes long)
  suitable for use in string.translate.  The strings frm and to
  must be of the same length. """
  return ""

def replace(str, old, new, maxsplit=None):
  """ replace (str, old, new[, maxsplit]) -> string
  
  Return a copy of string str with all occurrences of substring
  old replaced by new. If the optional argument maxsplit is
  given, only the first maxsplit occurrences are replaced. """
  return ""

def rfind(s, sub, start=None, end=None):
  """ rfind(s, sub [,start [,end]]) -> int
  
  Return the highest index in s where substring sub is found,
  such that sub is contained within s[start,end].  Optional
  arguments start and end are interpreted as in slice notation.
  
  Return -1 on failure. """
  return 1

def rstrip(s):
  """ rstrip(s) -> string
  
  Return a copy of the string s with trailing whitespace removed. """
  return ""

def split(s, sep=None, maxsplit=None):
  """ split(s [,sep [,maxsplit]]) -> list of strings
  splitfields(s [,sep [,maxsplit]]) -> list of strings
  
  Return a list of the words in the string s, using sep as the
  delimiter string.  If maxsplit is nonzero, splits into at most
  maxsplit words.  If sep is not specified, any whitespace string
  is a separator.  Maxsplit defaults to 0.
  
  (split and splitfields are synonymous) """
  return []

def splitfields(s, sep=None, maxsplit=None):
  """ split(s [,sep [,maxsplit]]) -> list of strings
  splitfields(s [,sep [,maxsplit]]) -> list of strings
  
  Return a list of the words in the string s, using sep as the
  delimiter string.  If maxsplit is nonzero, splits into at most
  maxsplit words.  If sep is not specified, any whitespace string
  is a separator.  Maxsplit defaults to 0.
  
  (split and splitfields are synonymous) """
  return []

def strip(s):
  """ strip(s) -> string
  
  Return a copy of the string s with leading and trailing
  whitespace removed. """
  return ""

def swapcase(s):
  """ swapcase(s) -> string
  
  Return a copy of the string s with upper case characters
  converted to lowercase and vice versa. """
  return ""

def translate(s, table, deletechars=None):
  """ translate(s,table [,deletechars]) -> string
  
  Return a copy of the string s, where all characters occurring
  in the optional argument deletechars are removed, and the
  remaining characters have been mapped through the given
  translation table, which must be a string of length 256. """
  return ""

def upper(s):
  """ upper(s) -> string
  
  Return a copy of the string s converted to uppercase. """
  return ""

uppercase = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
whitespace = """	

 """




#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is part of KDevelop
# Copyright 2010 Sven Brauch <svenbrauch@googlemail.com>
# Licensed under the GNU GPL
#

# This script once was used for parsing documents; that implementation is long gone.
# It's just kept for looking at the AST created by some code quickly; you can do this:
# echo 'foo = baz.bar(3, "str")' |./pythonpythonparser.py
# or similar to quickly see the AST for that code.

import ast
from xml.dom.minidom import Document
from lxml import etree
import types
import sys
import re

class KDevelopNodeVisitor(ast.NodeVisitor):
    basenode = etree.Element("pythonast")
    currentnode = None
    nodecnt = 0
    childNodeMap = {}
    
    def __init__(self, *arg, **args):
        super(KDevelopNodeVisitor, self).__init__(*arg, **args)
        self.currentnode = self.basenode
    
    def generic_visit(self, node):
        self.nodecnt += 1
        
        #self.childNodeMap[self.nodecnt] = node
        self.childNodeMap[node] = self.nodecnt
        
        node_xmlrepr = etree.Element(node.__class__.__name__ + "Ast")
        node_xmlrepr.set('nodecnt', str(self.nodecnt))
        self.currentnode.append(node_xmlrepr)
        
        save_currentnode = self.currentnode
        self.currentnode = node_xmlrepr
        
        fields = list(node._attributes)
        fields.extend(list(node._fields))
        
        searching_locally = []
        for field in fields:
            value = getattr(node, field)
            if type(value) not in [types.IntType, types.StringType, types.FloatType, types.BooleanType]:
                continue
            try:
                node_xmlrepr.set(field.lower(), str(value))
            except:
                sys.stderr.write("Warning: Invalid string literal replaced by empty string!\n")
                node_xmlrepr.set(field.lower(), "")


        super(KDevelopNodeVisitor, self).generic_visit(node)
        
        key = ''
        for field in fields:
            multiple_keys = []
            value = getattr(node, field)
            if type(value) not in [types.IntType, types.StringType, types.FloatType, types.BooleanType]:
                if type(value) == types.ListType:
                    for currentValue in value:
                        try:
                            multiple_keys.append(str(self.childNodeMap[currentValue]))
                        except KeyError:
                            sys.stderr.write("Warning: missing key on node " + str(node) + "\n")
                            multiple_keys.append('')
                    key = ','.join(multiple_keys)
                    node_xmlrepr.set("NRLST_" + field.lower(), str(key))
                else:
                    try:
                        key = self.childNodeMap[value]
                    except KeyError:
                        key = ''
                    node_xmlrepr.set("NR_" + field.lower(), str(key))
                
                
        self.currentnode = save_currentnode

f = sys.stdin.read()
v = KDevelopNodeVisitor()
try:
    parsetree = ast.parse(f)
except Exception as e:
    try:
        sys.stderr.write(str(e.lineno) + ':::' + str(e.offset))
        sys.stderr.write(":::" + str(type(e)).replace('<type \'exceptions.', '').replace('\'>', '') + ':::' + str(e.msg) + ": \"" + str(e.text).replace("\n", "") + "\"")
    except:
        sys.stderr.write('?:::?:::'+str(e)+':::?')
else:
    v.visit(parsetree)
    sys.stdout.write(etree.tostring(v.basenode, xml_declaration=True, pretty_print=True, encoding='UTF-8'))



# AUTO-GENERATED FILE -- DO NOT EDIT

""" Fast implementation of the datetime type. """

MAXYEAR = 9999
MINYEAR = 1
__package__ = None

class date(object):
  """ date(year, month, day) --> date object """

  def ctime(self):
    """ Return ctime() style string. """
    pass

  day = property(None, None, None,
                 )


  def fromordinal(self, int):
    """ int -> date corresponding to a proleptic Gregorian ordinal. """
    return None

  def fromtimestamp(self, timestamp):
    """ timestamp -> local date from a POSIX timestamp (like time.time()). """
    return None

  def isocalendar(self):
    """ Return a 3-tuple containing ISO year, week number, and weekday. """
    pass

  def isoformat(self):
    """ Return string in ISO 8601 format, YYYY-MM-DD. """
    pass

  def isoweekday(self, arg0):
    """ Return the day of the week represented by the date.
    Monday == 1 ... Sunday == 7 """
    return None

  max = None
  min = None
  month = property(None, None, None,
                   )


  def replace(self):
    """ Return date with new specified fields. """
    pass

  resolution = None

  def strftime(self, format):
    """ format -> strftime() style string. """
    return ""

  def timetuple(self):
    """ Return time tuple, compatible with time.localtime(). """
    pass

  def today(self, arg0):
    """ Current date or datetime:  same as self.__class__.fromtimestamp(time.time()). """
    pass

  def toordinal(self):
    """ Return proleptic Gregorian ordinal.  January 1 of year 1 is day 1. """
    pass

  def weekday(self, arg0):
    """ Return the day of the week represented by the date.
    Monday == 0 ... Sunday == 6 """
    return None

  year = property(None, None, None,
                  )


class datetime(date):
  """ datetime(year, month, day[, hour[, minute[, second[, microsecond[,tzinfo]]]]])
  
  The year, month and day arguments are required. tzinfo may be None, or an
  instance of a tzinfo subclass. The remaining arguments may be ints or longs.
   """

  def astimezone(self, tz):
    """ tz -> convert to local time in new timezone tz
     """
    return None

  def combine(self, date, time):
    """ date, time -> datetime with same date and time fields """
    return None

  def ctime(self):
    """ Return ctime() style string. """
    pass

  def date(self):
    """ Return date object with same year, month and day. """
    pass

  def dst(self):
    """ Return self.tzinfo.dst(self). """
    pass

  def fromordinal(self, int):
    """ int -> date corresponding to a proleptic Gregorian ordinal. """
    return None

  def fromtimestamp(self, timestamp, tz=None):
    """ timestamp[, tz] -> tz's local time from POSIX timestamp. """
    return None

  hour = property(None, None, None,
                  )


  def isoformat(self, sep=None):
    """ [sep] -> string in ISO 8601 format, YYYY-MM-DDTHH:MM:SS[.mmmmmm][+HH:MM].
    
    sep is used to separate the year from the time, and defaults to 'T'. """
    return ""

  max = None
  microsecond = property(None, None, None,
                         )

  min = None
  minute = property(None, None, None,
                    )


  def now(self, tz=None):
    """ [tz] -> new datetime with tz's local day and time. """
    return None

  def replace(self):
    """ Return datetime with new specified fields. """
    pass

  resolution = None
  second = property(None, None, None,
                    )


  def strptime(self, string, format):
    """ string, format -> new datetime parsed from a string (like time.strptime()). """
    return ""

  def time(self):
    """ Return time object with same time but with tzinfo=None. """
    pass

  def timetuple(self):
    """ Return time tuple, compatible with time.localtime(). """
    pass

  def timetz(self):
    """ Return time object with same time and tzinfo. """
    pass

  def today(self, arg0):
    """ Current date or datetime:  same as self.__class__.fromtimestamp(time.time()). """
    pass

  tzinfo = property(None, None, None,
                    )


  def tzname(self):
    """ Return self.tzinfo.tzname(self). """
    pass

  def utcfromtimestamp(self, timestamp):
    """ timestamp -> UTC datetime from a POSIX timestamp (like time.time()). """
    return None

  def utcnow(self):
    """ Return a new datetime representing UTC day and time. """
    pass

  def utcoffset(self):
    """ Return self.tzinfo.utcoffset(self). """
    pass

  def utctimetuple(self):
    """ Return UTC time tuple, compatible with time.localtime(). """
    pass

datetime_CAPI = None

class time(object):
  """ time([hour[, minute[, second[, microsecond[, tzinfo]]]]]) --> a time object
  
  All arguments are optional. tzinfo may be None, or an instance of
  a tzinfo subclass. The remaining arguments may be ints or longs.
   """

  def dst(self):
    """ Return self.tzinfo.dst(self). """
    pass

  hour = property(None, None, None,
                  )


  def isoformat(self):
    """ Return string in ISO 8601 format, HH:MM:SS[.mmmmmm][+HH:MM]. """
    pass

  max = None
  microsecond = property(None, None, None,
                         )

  min = None
  minute = property(None, None, None,
                    )


  def replace(self):
    """ Return time with new specified fields. """
    pass

  resolution = None
  second = property(None, None, None,
                    )


  def strftime(self, format):
    """ format -> strftime() style string. """
    return ""

  tzinfo = property(None, None, None,
                    )


  def tzname(self):
    """ Return self.tzinfo.tzname(self). """
    pass

  def utcoffset(self):
    """ Return self.tzinfo.utcoffset(self). """
    pass

class timedelta(object):
  """ Difference between two datetime values. """

  days = None
  max = None
  microseconds = None
  min = None
  resolution = None
  seconds = None

  def total_seconds(self):
    """ Total seconds in the duration. """
    pass

class tzinfo(object):
  """ Abstract base class for time zone info objects. """

  def dst(self, datetime):
    """ datetime -> DST offset in minutes east of UTC. """
    return None

  def fromutc(self, arg0):
    """ datetime in UTC -> datetime in local time. """
    return None

  def tzname(self, datetime):
    """ datetime -> string name of time zone. """
    return ""

  def utcoffset(self, datetime):
    """ datetime -> minutes east of UTC (negative for west of UTC). """
    return None




class pyqtSignal():
 def connect(self, targetSignal): pass
 def emit(self, *args): pass
from QtCore import *

class TerminalInterface():
    """"""
    def __init__(self):
        '''void TerminalInterface.__init__()'''
    def __init__(self):
        '''TerminalInterface TerminalInterface.__init__()'''
        return TerminalInterface()
    def sendInput(self, text):
        '''abstract void TerminalInterface.sendInput(QString text)'''
    def showShellInDir(self, dir):
        '''abstract void TerminalInterface.showShellInDir(QString dir)'''
    def startProgram(self, program, args):
        '''abstract void TerminalInterface.startProgram(QString program, QStringList args)'''


class TerminalInterfaceV2(TerminalInterface):
    """"""
    def __init__(self):
        '''void TerminalInterfaceV2.__init__()'''
    def __init__(self):
        '''TerminalInterfaceV2 TerminalInterfaceV2.__init__()'''
        return TerminalInterfaceV2()
    def foregroundProcessName(self):
        '''abstract QString TerminalInterfaceV2.foregroundProcessName()'''
        return QString()
    def foregroundProcessId(self):
        '''abstract int TerminalInterfaceV2.foregroundProcessId()'''
        return int()
    def terminalProcessId(self):
        '''abstract int TerminalInterfaceV2.terminalProcessId()'''
        return int()


class KTerminal():
    """"""
    def __init__(self, part):
        '''void KTerminal.__init__(KParts.Part part)'''
    def __init__(self):
        '''KTerminal KTerminal.__init__()'''
        return KTerminal()
    def terminalInterfaceV2(self):
        '''TerminalInterfaceV2 KTerminal.terminalInterfaceV2()'''
        return TerminalInterfaceV2()
    def terminalInterface(self):
        '''TerminalInterface KTerminal.terminalInterface()'''
        return TerminalInterface()





# Example file for correcting libraries.

from matplotlib.figure import Figure
from matplotlib.backend_bases import FigureCanvasBase

class class_FigureCanvasBase():
    def function___init__(self):
        l_figure = Figure()

class class_FigureManagerBase():
    def function___init__(self):
        l_canvas = FigureCanvasBase()

class class_NavigationToolbar2:
    def function___init__(self):
        l_canvas = FigureCanvasBase()

    def function_foo(self):
        returns = [Figure()]


# This file denotes the built-in python library. It is imported into every file which is parsed.
class Exception(object):
    pass

class object():
    def __init__(self): pass
    def __new__(self): pass
    def __del__(self): pass
    def __repr__(self): pass
    def __str__(self): pass
    def __lt__(self, other): pass
    def __gt__(self, other): pass
    def __le__(self, other): pass
    def __eq__(self, other): pass
    def __ne__(self, other): pass
    def __gt__(self, other): pass
    def __ge__(self, other): pass
    def __cmp__(self, other): pass
    def __hash__(self): pass
    def __nonzero__(self): pass
    def __unicode__(self): pass
    def __getattr__(self, name): pass
    def __setattr__(self, name, value): pass
    def __delattr__(self, name): pass
    def __getattribute__(self, name): pass
    def __get__(self, instance, owner): pass
    def __set__(self, instance, value): pass
    def __delete__(self, instance): pass
    def __instancecheck__(self, instance): pass
    def __subclasscheck__(self, subclass): pass
    def __call__(self): pass
    def __len__(self): pass
    def __getitem__(self, key): pass
    def __setitem__(self, key, value): pass
    def __delitem__(self, key): pass
    def __iter__(self): pass
    def __reversed__(self): pass
    def __contains__(self, item): pass
    def __getslice__(self, i, j): pass
    def __delslice__(self, i, j): pass
    def __add__(self, other): pass
    def __sub__(self, other): pass
    def __mul__(self, other): pass
    def __floordiv__(self, other): pass
    def __mod__(self, other): pass
    def __divmod__(self, other): pass
    def __pow__(self, other): pass
    def __lshift__(self, other): pass
    def __rshift__(self, other): pass
    def __and__(self, other): pass
    def __xor__(self, other): pass
    def __or__(self, other): pass
    def __div__(self, other): pass
    def __truediv__(self, other): pass
    def __radd__(self, other): pass
    def __rsub__(self, other): pass
    def __rmul__(self, other): pass
    def __rtruediv__(self, other): pass
    def __rfloordiv__(self, other): pass
    def __rmod__(self, other): pass
    def __rdivmod__(self, other): pass
    def __rpow__(self, other): pass
    def __rlshift__(self, other): pass
    def __rrshift__(self, other): pass
    def __rand__(self, other): pass
    def __rxor__(self, other): pass
    def __ror__(self, other): pass
    def __iadd__(self, other): pass
    def __isub__(self, other): pass
    def __imul__(self, other): pass
    def __idiv__(self, other): pass
    def __itruediv__(self, other): pass
    def __ifloordiv__(self, other): pass
    def __imod__(self, other): pass
    def __ipow__(self, other): pass
    def __ilshift__(self, other): pass
    def __irshift__(self, other): pass
    def __iand__(self, other): pass
    def __ixor__(self, other): pass
    def __ior__(self, other): pass
    def __neg__(self): pass
    def __pos__(self): pass
    def __abs__(self): pass
    def __invert__(self): pass
    def __complex__(self): pass
    def __int__(self): pass
    def __long__(self): pass
    def __float__(self): pass
    def __oct__(self): pass
    def __hex__(self): pass
    def __index__(self): pass
    def __coerce__(self, other): pass

    __class__ = str()

class basestring():
    pass

@TypeContainer
class list():
    @returnContentEqualsContentOf(0)
    def __init__(self, items):
        return []
    @addsTypeOfArg(1)
    def __setitem__(self, key, value): pass
    @getsType
    def __getitem__(self, key): pass
    @addsTypeOfArg(0)
    def append(self,obj): pass
    @addsTypeOfArgContent(0)
    def extend(self,obj): return None
    @addsTypeOfArg(0)
    def insert(self,i, x): return None
    @getsType
    def pop(self,i): return None
    @getsType
    def index(self,x): return 0
    def count(self,x): return 0
    @getsList
    def sort(self,): return []
    @getsList
    def reverse(self,): return []
    def remove(self, x): pass

class _io_TextIOWrapper():
    def close(self,): return None
    def flush(self,): return None
    def fileno(self,): return None
    def isatty(self,): return True
    def next(self,): return None
    def read(self,size = 0): return ""
    def readline(self,size = 0): return ""
    def readlines(self,sizehint = 0): return [""]
    def xreadlines(self,): return None
    def seek(self,offset, whence = 0): return None
    def tell(self,): return None
    def truncate(self,size = 0): return None
    def write(self,string): return None
    def writelines(self,sequence): return None
    closed = True
    errors = None
    mode = None
    name = ""
    newlines = ""
    softspace = True
    
@TypeContainer
@hasTypedKeys
class dict():
    @returnContentEqualsContentOf(0)
    def __init__(self, items):
        return {}
    @addsTypeOfArg(1)
    @addsKeyTypeOfArg(0)
    def __setitem__(self, key, value): pass
    @getsType
    def __getitem__(self, key): pass
    def clear(self,): return None
    def copy(self,): return {}
    @addsKeyTypeOfArgContent(0)
    @addsTypeOfArg(1)
    def fromkeys(self,seq, value = None): return {}
    @getsType
    def get(self,key, default = ""): return None
    def has_key(self,key): return True
    @getsListOfBoth
    def items(self,): return {}
    @getsListOfBoth
    def iteritems(self,): return []
    @getsListOfKeys
    def iterkeys(self,): return []
    @getsList
    def itervalues(self,): return []
    @getsListOfKeys
    def keys(self,): return []
    @getsType
    def pop(self,key, default = ""): return None
    @getsBoth
    def popitem(self,): return None
    def setdefault(self,key, default = ""): return None
    def update(self,other = None): return None
    @getsList
    def values(self,): return []
    def viewitems(self,): return None
    def viewkeys(self,): return None
    def viewvalues(self,): return None
    

class str():
    def __init__(self, obj):
        pass
    def __mod__(self, modulo):
        return str()
    def replace(self,before, after): return ""
    def capitalize(self,): return ""
    def center(self,width, fillchar = None): return ""
    def count(self,substring, start = 0, end = 0): return 0
    def decode(self,encoding = None, errors = None): return ""
    def endswith(self,suffix, start = 0, end = 0): return True
    def expandtabs(self,tabsize = 0): return ""
    def find(self,substring, start = 0, end = 0): return 0
    def format(self,*args, **kwargs): return ""
    def index(self,substring, start = 0, end = 0): return 0
    def isalnum(self,): return True
    def isalpha(self,): return True
    def isdigit(self,): return True
    def islower(self,): return True
    def isspace(self,): return True
    def istitle(self,): return True
    def isupper(self,): return True
    def join(self,iterable): return ""
    def ljust(self,width, fillchar = ""): return ""
    def lower(self,): return ""
    def lstrip(self,chars = ""): return ""
    def partition(self,seperator): return (self,"", "", "")
    def replace(self,old, new, count = 0): return ""
    def rfind(self,substring, start = 0, end = 0): return 0
    def rindex(self,substring, start = 0, end = 0): return 0
    def rjust(self,width, fillchar = ""): return ""
    def rpartition(self,seperator): return (self,"", "", "")
    def rsplit(self,seperator = "", maxsplit = 0): return []
    def rstrip(self,chars = ""): return ""
    def split(self,seperator = "", maxsplit = 0): return ["string"]
    def splitlines(self,keepends = False): return ["string"]
    def startswith(self,prefix, start = 0, end = 0): return True
    def strip(self,chars = ""): return ""
    def swapcase(self,): return ""
    def title(self,): return ""
    def translate(self,table, deletechars = ""): return ""
    def upper(self,): return ""
    def zfill(self,width): return ""

class float():
    def bit_length(self,): return 0
    def as_integer_ration(self,): return (self,0, 0)
    def is_integer(self,): return True
    def hex(self,): return 0x0
    def fromhex(self,s): return 0
    def __add__(self, other): return float()
    def __sub__(self, other): return float()
    def __mul__(self, other): return float()
    def __div__(self, other): return float()


class int():
    def __add__(self, other): return int()
    def __sub__(self, other): return int()
    def __mul__(self, other): return int()
    def __div__(self, other): return float()

class complex():
    real = 3
    imag = 5
    def __add__(self, other): return complex()
    def __sub__(self, other): return complex()
    def __mul__(self, other): return complex()
    def __div__(self, other): return complex()
    def __mod__(self, other): return complex()

class BaseException():
    args = ()

class NameError(BaseException):
    pass
class AttributeError(BaseException):
    pass
class IndexError(BaseException):
    pass
class IOError(BaseException):
    pass
class StandardError(BaseException):
    pass
class Exception(BaseException):
    pass
class BufferError(BaseException):
    pass
class LookupError(BaseException):
    pass
class EnvironmentError(BaseException):
    pass
class AssertionError(BaseException):
    pass
class EOFError(BaseException):
    pass
class FloatingPointError(BaseException):
    pass
class GeneratorExit(BaseException):
    pass
class ImportError(BaseException):
    pass
class KeyError(BaseException):
    pass
class MemoryError(BaseException):
    pass
class NotImplementedError(BaseException):
    pass
class OSError(BaseException):
    pass
class OverflowError(BaseException):
    pass
class ReferenceError(BaseException):
    pass
class RuntimeError(BaseException):
    pass
class StopIteration(BaseException):
    pass
class SyntaxError(BaseException):
    pass
class IndentationError(BaseException):
    pass
class TabError(BaseException):
    pass
class SystemError(BaseException):
    pass
class SystemExit(BaseException):
    pass
class TypeError(BaseException):
    pass
class UnboundLocalError(BaseException):
    pass
class UnicodeError(BaseException):
    pass
class UnicodeEncodeError(BaseException):
    pass
class UnicodeDecodeError(BaseException):
    pass
class UnicodeTranslateError(BaseException):
    pass
class ValueError(BaseException):
    pass
class ZeroDivisionError(BaseException):
    pass
class Warning():
    pass
class UserWarning(Warning):
    pass
class DeprecationWarning(Warning):
    pass
class KeyboardInterrupt(Exception):
    pass
class PendingDeprecationWarning(Warning):
    pass
class RuntimeWarning(Warning):
    pass
class FutureWarning(Warning):
    pass
class ImportWarning(Warning):
    pass
class UnicodeWarning(Warning):
    pass

@IndexedTypeContainer
class tuple():
    def __mul__(self, other): return tuple()
    pass

@TypeContainer
class set():
    @returnContentEqualsContentOf(0)
    def __init__(self, objects):
        pass
    def len(self): return 0
    def isdisjoint(self, other): return True
    def issubset(self, other): return True
    def issuperset(self, other): return True
    def union(self, other, *others): return set()
    def intersetion(self, other, *others): return set()
    def difference(self, other, *others): return set()
    def symmetric_difference(self, other): return set()
    def copy(self): return set()
    def update(self, other): pass
    def intersection_update(self, other, *others): pass
    def difference_update(self, other, *others): pass
    def symmetric_difference_update(self, other, *others): pass
    def add(self, elem): pass
    def remove(self, elem): pass
    def discard(self, elem): pass
    def pop(self): pass
    def clear(self): pass

def abs(x):
    """ Return the absolute value of a number. The argument may be a plain or long integer or a floating point number. If the argument is a complex number, its magnitude is returned."""
    return 0
def int(x): return 0
def all(iterable): return True
def any(iterable):
    """Return True if any element of the iterable is true. If the iterable is empty, return False."""
    return True
def bin(x):
    """Convert an integer number to a binary string. The result is a valid Python expression. If x is not a Python int object, it has to define an __index__() method that returns an integer."""
    return ""
def bool(x = False):
    """Convert a value to a Boolean, using the standard truth testing procedure. If x is false or omitted, this returns False; otherwise it returns True. bool is also a class, which is a subclass of int. Class bool cannot be subclassed further. Its only instances are False and True."""
    return True
def bytearray(source = None, encoding = None, errors = None):
    """Return a new array of bytes. The bytearray type is a mutable sequence of integers in the range 0 <= x < 256."""
    return []
def callable(object):
    """Return True if the object argument appears callable, False if not. If this returns true, it is still possible that a call fails, but if it is false, calling object will never succeed. Note that classes are callable (calling a class returns a new instance); class instances are callable if they have a __call__() method."""
    return True
def chr(i):
    """Return a string of one character whose ASCII code is the integer i. For example, chr(97) returns the string 'a'. This is the inverse of ord(). The argument must be in the range [0..255], inclusive; ValueError will be raised if i is outside that range. See also unichr()."""
    return ""
def classmethod(function):
    """Return a class method for function."""
    return None
def cmp(x, y):
    """Compare the two objects x and y and return an integer according to the outcome. The return value is negative if x < y, zero if x == y and strictly positive if x > y."""
    return 0
def compile(source, filename, mode, flags = None, dont_inherit = None):
    """Compile the source into a code or AST object. Code objects can be executed by an exec statement or evaluated by a call to eval(). source can either be a Unicode string, a Latin-1 encoded string or an AST object. Refer to the ast module documentation for information on how to work with AST objects."""
    return None
def delattr(obj, name):
    """This is a relative of setattr(). The arguments are an object and a string. The string must be the name of one of the object’s attributes. The function deletes the named attribute, provided the object allows it. For example, delattr(x, 'foobar') is equivalent to del x.foobar."""
    return None
def dir(obj = None):
    """Without arguments, return the list of names in the current local scope. With an argument, attempt to return a list of valid attributes for that object."""
    return {"string" : None}
def divmod(a, b):
    """Take two (non complex) numbers as arguments and return a pair of numbers consisting of their quotient and remainder when using long division."""
    return 0
def enumerate(sequence, start = 0):
    """Return an enumerate object. sequence must be a sequence, an iterator, or some other object which supports iteration."""
    return [(0, 0)]
def eval(expression, glob = None, loc = None):
    """The expression argument is parsed and evaluated as a Python expression (technically speaking, a condition list) using the globals and locals dictionaries as global and local namespace."""
    return None
def execfile(filename, glob = None, loc = None):
    """This function is similar to the exec statement, but parses a file instead of a string. It is different from the import statement in that it does not use the module administration — it reads the file unconditionally and does not create a new module. """
    return None
def file(filename, mode = None, bufsize = None):
    """Constructor function for the file type, described further in section File Objects."""
    return _io_TextIOWrapper()
def filter(function, iterable):
    """Construct a list from those elements of iterable for which function returns true. iterable may be either a sequence, a container which supports iteration, or an iterator. If iterable is a string or a tuple, the result also has that type; otherwise it is always a list. If function is None, the identity function is assumed, that is, all elements of iterable that are false are removed."""
    return []
def float(x = 0):
    """Convert a string or a number to floating point."""
    return 0.0
def format(value, format_spec = None):
    """Convert a value to a “formatted” representation, as controlled by format_spec."""
    return ""
def frozenset(iterable = None):
    """Return a new frozenset object, optionally with elements taken from iterable."""
    return set()
def getattr(obj, name, default = None):
    """Return the value of the named attribute of object. name must be a string."""
    return None
def globals():
    """Return a dictionary representing the current global symbol table."""
    return {}
def hasattr(obj, name):
    """The arguments are an object and a string. The result is True if the string is the name of one of the object’s attributes, False if not. (This is implemented by calling getattr(object, name) and seeing whether it raises an exception or not.)"""
    return bool
def hash(obj):
    """Return the hash value of the object (if it has one)."""
    return 0
def hex(x):
    """Convert an integer number (of any size) to a hexadecimal string."""
    return 0x0
def id(obj):
    """Return the “identity” of an object. This is an integer (or long integer) which is guaranteed to be unique and constant for this object during its lifetime. Two objects with non-overlapping lifetimes may have the same id() value."""
    return 0
def input(prompt = None):
    """Equivalent to eval(raw_input(prompt))."""
    return None
def isinstance(obj, cls):
    """Return true if the object argument is an instance of the classinfo argument, or of a (direct, indirect or virtual) subclass thereof."""
    return True
def issubclass(cls, info):
    """Return true if class is a subclass (direct, indirect or virtual) of classinfo."""
    return True
@returnContentEqualsContentOf(0)
def iter(o, s = None):
    """Return an iterator object."""
    return []
def len(s):
    """Return the length (the number of items) of an object. The argument may be a sequence (string, tuple or list) or a mapping (dictionary)."""
    return 0
def locals():
    """Update and return a dictionary representing the current local symbol table.""" 
    return {}
def long(x = None, base = None):
    """Convert a string or number to a long integer."""
    return 0L
def map(func, iterab):
    """Apply function to every item of iterable and return a list of the results."""
    return []
def max(lst, args = None, key = None):
    """Return the largest item in an iterable or the largest of two or more arguments."""
    return 0
def memoryview(obj):
    """Return a “memory view” object created from the given argument."""
    return None
def min(lst, default = None):
    """Return the smallest item in an iterable or the smallest of two or more arguments."""
    return 0
def next(iterator, default = None):
    """Retrieve the next item from the iterator by calling its next() method."""
    return iterator[0]
def oct(x):
    """Convert an integer number (of any size) to an octal string."""
    return 0o0
def open(filename, mode = None, bufsize = None):
    """Open a file, returning an object of the file type described in section File Objects."""
    return _io_TextIOWrapper()
def ord(c):
    """Given a string of length one, return an integer representing the Unicode code point of the character when the argument is a unicode object, or the value of the byte when the argument is an 8-bit string."""
    return 0
def pow(x, y, z = 0):
    """Return x to the power y; if z is present, return x to the power y, modulo z."""
    return 0.0
def property(fget = 0, fset = 0, fdel = 0, doc = 0):
    """Return a property attribute for new-style classes (classes that derive from object)."""
    return 0
def range(start = 0, stop = 0, step = 0):
    """This is a versatile function to create lists containing arithmetic progressions. It is most often used in for loops. The arguments must be plain integers."""
    return [0]
def raw_input(prompt = ""):
    """ The function then reads a line from input, converts it to a string (stripping a trailing newline), and returns that."""
    return ""
def reduce(function, iterable, init = None):
    """Apply function of two arguments cumulatively to the items of iterable, from left to right, so as to reduce the iterable to a single value."""
    return None
def reload(module):
    """Reload a previously imported module."""
    return None
def repr(object):
    """Return a string containing a printable representation of an object."""
    return ""
@returnContentEqualsContentOf(0)
def reversed(seq):
    """Return a reverse iterator."""
    return None
def round(x, n=0):
    """Return the floating point value number rounded to ndigits digits after the decimal point."""
    return 0.0
def setattr(obj, name, value):
    """This is the counterpart of getattr(). The arguments are an object, a string and an arbitrary value. The string may name an existing attribute or a new attribute. The function assigns the value to the attribute, provided the object allows it."""
    return None
def slice(start = 0, stop = 0, step = 0):
    """Return a slice object representing the set of indices specified by range(start, stop, step)."""
    return slice()
def sorted(iterable, cmpre = None, key = None, reverse = False):
    """Return a new sorted list from the items in iterable."""
    return []
def staticmethod(function):
    """Return a static method for function."""
    return function
def sum(iterable):
    """Sums start and the items of an iterable from left to right and returns the total."""
    return 0.0
def super(_type, obj = None):
    """Return a proxy object that delegates method calls to a parent or sibling class of type."""
    return None
def tuple(iterable = None):
    """Return a tuple whose items are the same and in the same order as iterable‘s items."""
    return ()
def type(object):
    """With one argument, return the type of an object."""
    return object
def unichr(i):
    """Return the Unicode string of one character whose Unicode code is the integer i."""
    return ""
def unicode(obj = None, encoding = None, errors = None):
    """Return the Unicode string version of object."""
    return ""
def vars(obj):
    """Return the __dict__ attribute for a module, class, instance, or any other object with a __dict__ attribute."""
    return None
def xrange(start = 0, stop = 0, step = 0):
    """This function is very similar to range(), but returns an xrange object instead of a list."""
    return [0]
def zip(iterable = None):
    """This function returns a list of tuples, where the i-th tuple contains the i-th element from each of the argument sequences or iterables."""
    return []
def __import__(name, globa = None, loca = None, fromlist = None, level = 0):
    """This function is invoked by the import statement. It can be replaced (by importing the __builtin__ module and assigning to __builtin__.__import__) in order to change semantics of the import statement, but nowadays it is usually simpler to use import hooks (see PEP 302)."""
    return None
def exit(status): return None
__name__ = "none"
__file__ = "none"
__doc__ = "none"
__package__ = "none"




########################################################################
# Copyright (c) 2000, BeOpen.com.
# Copyright (c) 1995-2000, Corporation for National Research Initiatives.
# Copyright (c) 1990-1995, Stichting Mathematisch Centrum.
# All rights reserved.
#
# See the file "Misc/COPYRIGHT" for information on usage and
# redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
########################################################################

# Python script to parse cstubs file for gl and generate C stubs.
# usage: python cgen.py <cstubs >glmodule.c
#
# NOTE: You  must first make a python binary without the "GL" option
#       before you can run this, when building Python for the first time.
#       See comments in the Makefile.
#
# XXX BUG return arrays generate wrong code
# XXX need to change error returns into gotos to free mallocked arrays
from warnings import warnpy3k
warnpy3k("the cgen module has been removed in Python 3.0", stacklevel=2)
del warnpy3k


import string
import sys


# Function to print to stderr
#
def err(*args):
    savestdout = sys.stdout
    try:
        sys.stdout = sys.stderr
        for i in args:
            print i,
        print
    finally:
        sys.stdout = savestdout


# The set of digits that form a number
#
digits = '0123456789'


# Function to extract a string of digits from the front of the string.
# Returns the leading string of digits and the remaining string.
# If no number is found, returns '' and the original string.
#
def getnum(s):
    n = ''
    while s and s[0] in digits:
        n = n + s[0]
        s = s[1:]
    return n, s


# Function to check if a string is a number
#
def isnum(s):
    if not s: return False
    for c in s:
        if not c in digits: return False
    return True


# Allowed function return types
#
return_types = ['void', 'short', 'long']


# Allowed function argument types
#
arg_types = ['char', 'string', 'short', 'u_short', 'float', 'long', 'double']


# Need to classify arguments as follows
#       simple input variable
#       simple output variable
#       input array
#       output array
#       input giving size of some array
#
# Array dimensions can be specified as follows
#       constant
#       argN
#       constant * argN
#       retval
#       constant * retval
#
# The dimensions given as constants * something are really
# arrays of points where points are 2- 3- or 4-tuples
#
# We have to consider three lists:
#       python input arguments
#       C stub arguments (in & out)
#       python output arguments (really return values)
#
# There is a mapping from python input arguments to the input arguments
# of the C stub, and a further mapping from C stub arguments to the
# python return values


# Exception raised by checkarg() and generate()
#
arg_error = 'bad arg'


# Function to check one argument.
# Arguments: the type and the arg "name" (really mode plus subscript).
# Raises arg_error if something's wrong.
# Return type, mode, factor, rest of subscript; factor and rest may be empty.
#
def checkarg(type, arg):
    #
    # Turn "char *x" into "string x".
    #
    if type == 'char' and arg[0] == '*':
        type = 'string'
        arg = arg[1:]
    #
    # Check that the type is supported.
    #
    if type not in arg_types:
        raise arg_error, ('bad type', type)
    if type[:2] == 'u_':
        type = 'unsigned ' + type[2:]
    #
    # Split it in the mode (first character) and the rest.
    #
    mode, rest = arg[:1], arg[1:]
    #
    # The mode must be 's' for send (= input) or 'r' for return argument.
    #
    if mode not in ('r', 's'):
        raise arg_error, ('bad arg mode', mode)
    #
    # Is it a simple argument: if so, we are done.
    #
    if not rest:
        return type, mode, '', ''
    #
    # Not a simple argument; must be an array.
    # The 'rest' must be a subscript enclosed in [ and ].
    # The subscript must be one of the following forms,
    # otherwise we don't handle it (where N is a number):
    #       N
    #       argN
    #       retval
    #       N*argN
    #       N*retval
    #
    if rest[:1] <> '[' or rest[-1:] <> ']':
        raise arg_error, ('subscript expected', rest)
    sub = rest[1:-1]
    #
    # Is there a leading number?
    #
    num, sub = getnum(sub)
    if num:
        # There is a leading number
        if not sub:
            # The subscript is just a number
            return type, mode, num, ''
        if sub[:1] == '*':
            # There is a factor prefix
            sub = sub[1:]
        else:
            raise arg_error, ('\'*\' expected', sub)
    if sub == 'retval':
        # size is retval -- must be a reply argument
        if mode <> 'r':
            raise arg_error, ('non-r mode with [retval]', mode)
    elif not isnum(sub) and (sub[:3] <> 'arg' or not isnum(sub[3:])):
        raise arg_error, ('bad subscript', sub)
    #
    return type, mode, num, sub


# List of functions for which we have generated stubs
#
functions = []


# Generate the stub for the given function, using the database of argument
# information build by successive calls to checkarg()
#
def generate(type, func, database):
    #
    # Check that we can handle this case:
    # no variable size reply arrays yet
    #
    n_in_args = 0
    n_out_args = 0
    #
    for a_type, a_mode, a_factor, a_sub in database:
        if a_mode == 's':
            n_in_args = n_in_args + 1
        elif a_mode == 'r':
            n_out_args = n_out_args + 1
        else:
            # Can't happen
            raise arg_error, ('bad a_mode', a_mode)
        if (a_mode == 'r' and a_sub) or a_sub == 'retval':
            err('Function', func, 'too complicated:',
                a_type, a_mode, a_factor, a_sub)
            print '/* XXX Too complicated to generate code for */'
            return
    #
    functions.append(func)
    #
    # Stub header
    #
    print
    print 'static PyObject *'
    print 'gl_' + func + '(self, args)'
    print '\tPyObject *self;'
    print '\tPyObject *args;'
    print '{'
    #
    # Declare return value if any
    #
    if type <> 'void':
        print '\t' + type, 'retval;'
    #
    # Declare arguments
    #
    for i in range(len(database)):
        a_type, a_mode, a_factor, a_sub = database[i]
        print '\t' + a_type,
        brac = ket = ''
        if a_sub and not isnum(a_sub):
            if a_factor:
                brac = '('
                ket = ')'
            print brac + '*',
        print 'arg' + repr(i+1) + ket,
        if a_sub and isnum(a_sub):
            print '[', a_sub, ']',
        if a_factor:
            print '[', a_factor, ']',
        print ';'
    #
    # Find input arguments derived from array sizes
    #
    for i in range(len(database)):
        a_type, a_mode, a_factor, a_sub = database[i]
        if a_mode == 's' and a_sub[:3] == 'arg' and isnum(a_sub[3:]):
            # Sending a variable-length array
            n = eval(a_sub[3:])
            if 1 <= n <= len(database):
                b_type, b_mode, b_factor, b_sub = database[n-1]
                if b_mode == 's':
                    database[n-1] = b_type, 'i', a_factor, repr(i)
                    n_in_args = n_in_args - 1
    #
    # Assign argument positions in the Python argument list
    #
    in_pos = []
    i_in = 0
    for i in range(len(database)):
        a_type, a_mode, a_factor, a_sub = database[i]
        if a_mode == 's':
            in_pos.append(i_in)
            i_in = i_in + 1
        else:
            in_pos.append(-1)
    #
    # Get input arguments
    #
    for i in range(len(database)):
        a_type, a_mode, a_factor, a_sub = database[i]
        if a_type[:9] == 'unsigned ':
            xtype = a_type[9:]
        else:
            xtype = a_type
        if a_mode == 'i':
            #
            # Implicit argument;
            # a_factor is divisor if present,
            # a_sub indicates which arg (`database index`)
            #
            j = eval(a_sub)
            print '\tif',
            print '(!geti' + xtype + 'arraysize(args,',
            print repr(n_in_args) + ',',
            print repr(in_pos[j]) + ',',
            if xtype <> a_type:
                print '('+xtype+' *)',
            print '&arg' + repr(i+1) + '))'
            print '\t\treturn NULL;'
            if a_factor:
                print '\targ' + repr(i+1),
                print '= arg' + repr(i+1),
                print '/', a_factor + ';'
        elif a_mode == 's':
            if a_sub and not isnum(a_sub):
                # Allocate memory for varsize array
                print '\tif ((arg' + repr(i+1), '=',
                if a_factor:
                    print '('+a_type+'(*)['+a_factor+'])',
                print 'PyMem_NEW(' + a_type, ',',
                if a_factor:
                    print a_factor, '*',
                print a_sub, ')) == NULL)'
                print '\t\treturn PyErr_NoMemory();'
            print '\tif',
            if a_factor or a_sub: # Get a fixed-size array array
                print '(!geti' + xtype + 'array(args,',
                print repr(n_in_args) + ',',
                print repr(in_pos[i]) + ',',
                if a_factor: print a_factor,
                if a_factor and a_sub: print '*',
                if a_sub: print a_sub,
                print ',',
                if (a_sub and a_factor) or xtype <> a_type:
                    print '('+xtype+' *)',
                print 'arg' + repr(i+1) + '))'
            else: # Get a simple variable
                print '(!geti' + xtype + 'arg(args,',
                print repr(n_in_args) + ',',
                print repr(in_pos[i]) + ',',
                if xtype <> a_type:
                    print '('+xtype+' *)',
                print '&arg' + repr(i+1) + '))'
            print '\t\treturn NULL;'
    #
    # Begin of function call
    #
    if type <> 'void':
        print '\tretval =', func + '(',
    else:
        print '\t' + func + '(',
    #
    # Argument list
    #
    for i in range(len(database)):
        if i > 0: print ',',
        a_type, a_mode, a_factor, a_sub = database[i]
        if a_mode == 'r' and not a_factor:
            print '&',
        print 'arg' + repr(i+1),
    #
    # End of function call
    #
    print ');'
    #
    # Free varsize arrays
    #
    for i in range(len(database)):
        a_type, a_mode, a_factor, a_sub = database[i]
        if a_mode == 's' and a_sub and not isnum(a_sub):
            print '\tPyMem_DEL(arg' + repr(i+1) + ');'
    #
    # Return
    #
    if n_out_args:
        #
        # Multiple return values -- construct a tuple
        #
        if type <> 'void':
            n_out_args = n_out_args + 1
        if n_out_args == 1:
            for i in range(len(database)):
                a_type, a_mode, a_factor, a_sub = database[i]
                if a_mode == 'r':
                    break
            else:
                raise arg_error, 'expected r arg not found'
            print '\treturn',
            print mkobject(a_type, 'arg' + repr(i+1)) + ';'
        else:
            print '\t{ PyObject *v = PyTuple_New(',
            print n_out_args, ');'
            print '\t  if (v == NULL) return NULL;'
            i_out = 0
            if type <> 'void':
                print '\t  PyTuple_SetItem(v,',
                print repr(i_out) + ',',
                print mkobject(type, 'retval') + ');'
                i_out = i_out + 1
            for i in range(len(database)):
                a_type, a_mode, a_factor, a_sub = database[i]
                if a_mode == 'r':
                    print '\t  PyTuple_SetItem(v,',
                    print repr(i_out) + ',',
                    s = mkobject(a_type, 'arg' + repr(i+1))
                    print s + ');'
                    i_out = i_out + 1
            print '\t  return v;'
            print '\t}'
    else:
        #
        # Simple function return
        # Return None or return value
        #
        if type == 'void':
            print '\tPy_INCREF(Py_None);'
            print '\treturn Py_None;'
        else:
            print '\treturn', mkobject(type, 'retval') + ';'
    #
    # Stub body closing brace
    #
    print '}'


# Subroutine to return a function call to mknew<type>object(<arg>)
#
def mkobject(type, arg):
    if type[:9] == 'unsigned ':
        type = type[9:]
        return 'mknew' + type + 'object((' + type + ') ' + arg + ')'
    return 'mknew' + type + 'object(' + arg + ')'


defined_archs = []

# usage: cgen [ -Dmach ... ] [ file ]
for arg in sys.argv[1:]:
    if arg[:2] == '-D':
        defined_archs.append(arg[2:])
    else:
        # Open optional file argument
        sys.stdin = open(arg, 'r')


# Input line number
lno = 0


# Input is divided in two parts, separated by a line containing '%%'.
#       <part1>         -- literally copied to stdout
#       <part2>         -- stub definitions

# Variable indicating the current input part.
#
part = 1

# Main loop over the input
#
while 1:
    try:
        line = raw_input()
    except EOFError:
        break
    #
    lno = lno+1
    words = string.split(line)
    #
    if part == 1:
        #
        # In part 1, copy everything literally
        # except look for a line of just '%%'
        #
        if words == ['%%']:
            part = part + 1
        else:
            #
            # Look for names of manually written
            # stubs: a single percent followed by the name
            # of the function in Python.
            # The stub name is derived by prefixing 'gl_'.
            #
            if words and words[0][0] == '%':
                func = words[0][1:]
                if (not func) and words[1:]:
                    func = words[1]
                if func:
                    functions.append(func)
            else:
                print line
        continue
    if not words:
        continue                # skip empty line
    elif words[0] == 'if':
        # if XXX rest
        # if !XXX rest
        if words[1][0] == '!':
            if words[1][1:] in defined_archs:
                continue
        elif words[1] not in defined_archs:
            continue
        words = words[2:]
    if words[0] == '#include':
        print line
    elif words[0][:1] == '#':
        pass                    # ignore comment
    elif words[0] not in return_types:
        err('Line', lno, ': bad return type :', words[0])
    elif len(words) < 2:
        err('Line', lno, ': no funcname :', line)
    else:
        if len(words) % 2 <> 0:
            err('Line', lno, ': odd argument list :', words[2:])
        else:
            database = []
            try:
                for i in range(2, len(words), 2):
                    x = checkarg(words[i], words[i+1])
                    database.append(x)
                print
                print '/*',
                for w in words: print w,
                print '*/'
                generate(words[0], words[1], database)
            except arg_error, msg:
                err('Line', lno, ':', msg)


print
print 'static struct PyMethodDef gl_methods[] = {'
for func in functions:
    print '\t{"' + func + '", gl_' + func + '},'
print '\t{NULL, NULL} /* Sentinel */'
print '};'
print
print 'void'
print 'initgl()'
print '{'
print '\t(void) Py_InitModule("gl", gl_methods);'
print '}'



"""Utility functions for copying and archiving files and directory trees.

XXX The functions here don't copy the resource fork or other metadata on Mac.

"""

import os
import sys
import stat
from os.path import abspath
import fnmatch
import collections
import errno

try:
    from pwd import getpwnam
except ImportError:
    getpwnam = None

try:
    from grp import getgrnam
except ImportError:
    getgrnam = None

__all__ = ["copyfileobj", "copyfile", "copymode", "copystat", "copy", "copy2",
           "copytree", "move", "rmtree", "Error", "SpecialFileError",
           "ExecError", "make_archive", "get_archive_formats",
           "register_archive_format", "unregister_archive_format"]

class Error(EnvironmentError):
    pass

class SpecialFileError(EnvironmentError):
    """Raised when trying to do a kind of operation (e.g. copying) which is
    not supported on a special file (e.g. a named pipe)"""

class ExecError(EnvironmentError):
    """Raised when a command could not be executed"""

try:
    WindowsError
except NameError:
    WindowsError = None

def copyfileobj(fsrc, fdst, length=16*1024):
    """copy data from file-like object fsrc to file-like object fdst"""
    while 1:
        buf = fsrc.read(length)
        if not buf:
            break
        fdst.write(buf)

def _samefile(src, dst):
    # Macintosh, Unix.
    if hasattr(os.path, 'samefile'):
        try:
            return os.path.samefile(src, dst)
        except OSError:
            return False

    # All other platforms: check for same pathname.
    return (os.path.normcase(os.path.abspath(src)) ==
            os.path.normcase(os.path.abspath(dst)))

def copyfile(src, dst):
    """Copy data from src to dst"""
    if _samefile(src, dst):
        raise Error("`%s` and `%s` are the same file" % (src, dst))

    for fn in [src, dst]:
        try:
            st = os.stat(fn)
        except OSError:
            # File most likely does not exist
            pass
        else:
            # XXX What about other special files? (sockets, devices...)
            if stat.S_ISFIFO(st.st_mode):
                raise SpecialFileError("`%s` is a named pipe" % fn)

    with open(src, 'rb') as fsrc:
        with open(dst, 'wb') as fdst:
            copyfileobj(fsrc, fdst)

def copymode(src, dst):
    """Copy mode bits from src to dst"""
    if hasattr(os, 'chmod'):
        st = os.stat(src)
        mode = stat.S_IMODE(st.st_mode)
        os.chmod(dst, mode)

def copystat(src, dst):
    """Copy all stat info (mode bits, atime, mtime, flags) from src to dst"""
    st = os.stat(src)
    mode = stat.S_IMODE(st.st_mode)
    if hasattr(os, 'utime'):
        os.utime(dst, (st.st_atime, st.st_mtime))
    if hasattr(os, 'chmod'):
        os.chmod(dst, mode)
    if hasattr(os, 'chflags') and hasattr(st, 'st_flags'):
        try:
            os.chflags(dst, st.st_flags)
        except OSError, why:
            if (not hasattr(errno, 'EOPNOTSUPP') or
                why.errno != errno.EOPNOTSUPP):
                raise

def copy(src, dst):
    """Copy data and mode bits ("cp src dst").

    The destination may be a directory.

    """
    if os.path.isdir(dst):
        dst = os.path.join(dst, os.path.basename(src))
    copyfile(src, dst)
    copymode(src, dst)

def copy2(src, dst):
    """Copy data and all stat info ("cp -p src dst").

    The destination may be a directory.

    """
    if os.path.isdir(dst):
        dst = os.path.join(dst, os.path.basename(src))
    copyfile(src, dst)
    copystat(src, dst)

def ignore_patterns(*patterns):
    """Function that can be used as copytree() ignore parameter.

    Patterns is a sequence of glob-style patterns
    that are used to exclude files"""
    def _ignore_patterns(path, names):
        ignored_names = []
        for pattern in patterns:
            ignored_names.extend(fnmatch.filter(names, pattern))
        return set(ignored_names)
    return _ignore_patterns

def copytree(src, dst, symlinks=False, ignore=None):
    """Recursively copy a directory tree using copy2().

    The destination directory must not already exist.
    If exception(s) occur, an Error is raised with a list of reasons.

    If the optional symlinks flag is true, symbolic links in the
    source tree result in symbolic links in the destination tree; if
    it is false, the contents of the files pointed to by symbolic
    links are copied.

    The optional ignore argument is a callable. If given, it
    is called with the `src` parameter, which is the directory
    being visited by copytree(), and `names` which is the list of
    `src` contents, as returned by os.listdir():

        callable(src, names) -> ignored_names

    Since copytree() is called recursively, the callable will be
    called once for each directory that is copied. It returns a
    list of names relative to the `src` directory that should
    not be copied.

    XXX Consider this example code rather than the ultimate tool.

    """
    names = os.listdir(src)
    if ignore is not None:
        ignored_names = ignore(src, names)
    else:
        ignored_names = set()

    os.makedirs(dst)
    errors = []
    for name in names:
        if name in ignored_names:
            continue
        srcname = os.path.join(src, name)
        dstname = os.path.join(dst, name)
        try:
            if symlinks and os.path.islink(srcname):
                linkto = os.readlink(srcname)
                os.symlink(linkto, dstname)
            elif os.path.isdir(srcname):
                copytree(srcname, dstname, symlinks, ignore)
            else:
                # Will raise a SpecialFileError for unsupported file types
                copy2(srcname, dstname)
        # catch the Error from the recursive copytree so that we can
        # continue with other files
        except Error, err:
            errors.extend(err.args[0])
        except EnvironmentError, why:
            errors.append((srcname, dstname, str(why)))
    try:
        copystat(src, dst)
    except OSError, why:
        if WindowsError is not None and isinstance(why, WindowsError):
            # Copying file access times may fail on Windows
            pass
        else:
            errors.extend((src, dst, str(why)))
    if errors:
        raise Error, errors

def rmtree(path, ignore_errors=False, onerror=None):
    """Recursively delete a directory tree.

    If ignore_errors is set, errors are ignored; otherwise, if onerror
    is set, it is called to handle the error with arguments (func,
    path, exc_info) where func is os.listdir, os.remove, or os.rmdir;
    path is the argument to that function that caused it to fail; and
    exc_info is a tuple returned by sys.exc_info().  If ignore_errors
    is false and onerror is None, an exception is raised.

    """
    if ignore_errors:
        def onerror(*args):
            pass
    elif onerror is None:
        def onerror(*args):
            raise
    try:
        if os.path.islink(path):
            # symlinks to directories are forbidden, see bug #1669
            raise OSError("Cannot call rmtree on a symbolic link")
    except OSError:
        onerror(os.path.islink, path, sys.exc_info())
        # can't continue even if onerror hook returns
        return
    names = []
    try:
        names = os.listdir(path)
    except os.error, err:
        onerror(os.listdir, path, sys.exc_info())
    for name in names:
        fullname = os.path.join(path, name)
        try:
            mode = os.lstat(fullname).st_mode
        except os.error:
            mode = 0
        if stat.S_ISDIR(mode):
            rmtree(fullname, ignore_errors, onerror)
        else:
            try:
                os.remove(fullname)
            except os.error, err:
                onerror(os.remove, fullname, sys.exc_info())
    try:
        os.rmdir(path)
    except os.error:
        onerror(os.rmdir, path, sys.exc_info())


def _basename(path):
    # A basename() variant which first strips the trailing slash, if present.
    # Thus we always get the last component of the path, even for directories.
    return os.path.basename(path.rstrip(os.path.sep))

def move(src, dst):
    """Recursively move a file or directory to another location. This is
    similar to the Unix "mv" command.

    If the destination is a directory or a symlink to a directory, the source
    is moved inside the directory. The destination path must not already
    exist.

    If the destination already exists but is not a directory, it may be
    overwritten depending on os.rename() semantics.

    If the destination is on our current filesystem, then rename() is used.
    Otherwise, src is copied to the destination and then removed.
    A lot more could be done here...  A look at a mv.c shows a lot of
    the issues this implementation glosses over.

    """
    real_dst = dst
    if os.path.isdir(dst):
        real_dst = os.path.join(dst, _basename(src))
        if os.path.exists(real_dst):
            raise Error, "Destination path '%s' already exists" % real_dst
    try:
        os.rename(src, real_dst)
    except OSError:
        if os.path.isdir(src):
            if _destinsrc(src, dst):
                raise Error, "Cannot move a directory '%s' into itself '%s'." % (src, dst)
            copytree(src, real_dst, symlinks=True)
            rmtree(src)
        else:
            copy2(src, real_dst)
            os.unlink(src)

def _destinsrc(src, dst):
    src = abspath(src)
    dst = abspath(dst)
    if not src.endswith(os.path.sep):
        src += os.path.sep
    if not dst.endswith(os.path.sep):
        dst += os.path.sep
    return dst.startswith(src)

def _get_gid(name):
    """Returns a gid, given a group name."""
    if getgrnam is None or name is None:
        return None
    try:
        result = getgrnam(name)
    except KeyError:
        result = None
    if result is not None:
        return result[2]
    return None

def _get_uid(name):
    """Returns an uid, given a user name."""
    if getpwnam is None or name is None:
        return None
    try:
        result = getpwnam(name)
    except KeyError:
        result = None
    if result is not None:
        return result[2]
    return None

def _make_tarball(base_name, base_dir, compress="gzip", verbose=0, dry_run=0,
                  owner=None, group=None, logger=None):
    """Create a (possibly compressed) tar file from all the files under
    'base_dir'.

    'compress' must be "gzip" (the default), "bzip2", or None.

    'owner' and 'group' can be used to define an owner and a group for the
    archive that is being built. If not provided, the current owner and group
    will be used.

    The output tar file will be named 'base_dir' +  ".tar", possibly plus
    the appropriate compression extension (".gz", or ".bz2").

    Returns the output filename.
    """
    tar_compression = {'gzip': 'gz', 'bzip2': 'bz2', None: ''}
    compress_ext = {'gzip': '.gz', 'bzip2': '.bz2'}

    # flags for compression program, each element of list will be an argument
    if compress is not None and compress not in compress_ext.keys():
        raise ValueError, \
              ("bad value for 'compress': must be None, 'gzip' or 'bzip2'")

    archive_name = base_name + '.tar' + compress_ext.get(compress, '')
    archive_dir = os.path.dirname(archive_name)

    if not os.path.exists(archive_dir):
        logger.info("creating %s" % archive_dir)
        if not dry_run:
            os.makedirs(archive_dir)


    # creating the tarball
    import tarfile  # late import so Python build itself doesn't break

    if logger is not None:
        logger.info('Creating tar archive')

    uid = _get_uid(owner)
    gid = _get_gid(group)

    def _set_uid_gid(tarinfo):
        if gid is not None:
            tarinfo.gid = gid
            tarinfo.gname = group
        if uid is not None:
            tarinfo.uid = uid
            tarinfo.uname = owner
        return tarinfo

    if not dry_run:
        tar = tarfile.open(archive_name, 'w|%s' % tar_compression[compress])
        try:
            tar.add(base_dir, filter=_set_uid_gid)
        finally:
            tar.close()

    return archive_name

def _call_external_zip(base_dir, zip_filename, verbose=False, dry_run=False):
    # XXX see if we want to keep an external call here
    if verbose:
        zipoptions = "-r"
    else:
        zipoptions = "-rq"
    from distutils.errors import DistutilsExecError
    from distutils.spawn import spawn
    try:
        spawn(["zip", zipoptions, zip_filename, base_dir], dry_run=dry_run)
    except DistutilsExecError:
        # XXX really should distinguish between "couldn't find
        # external 'zip' command" and "zip failed".
        raise ExecError, \
            ("unable to create zip file '%s': "
            "could neither import the 'zipfile' module nor "
            "find a standalone zip utility") % zip_filename

def _make_zipfile(base_name, base_dir, verbose=0, dry_run=0, logger=None):
    """Create a zip file from all the files under 'base_dir'.

    The output zip file will be named 'base_dir' + ".zip".  Uses either the
    "zipfile" Python module (if available) or the InfoZIP "zip" utility
    (if installed and found on the default search path).  If neither tool is
    available, raises ExecError.  Returns the name of the output zip
    file.
    """
    zip_filename = base_name + ".zip"
    archive_dir = os.path.dirname(base_name)

    if not os.path.exists(archive_dir):
        if logger is not None:
            logger.info("creating %s", archive_dir)
        if not dry_run:
            os.makedirs(archive_dir)

    # If zipfile module is not available, try spawning an external 'zip'
    # command.
    try:
        import zipfile
    except ImportError:
        zipfile = None

    if zipfile is None:
        _call_external_zip(base_dir, zip_filename, verbose, dry_run)
    else:
        if logger is not None:
            logger.info("creating '%s' and adding '%s' to it",
                        zip_filename, base_dir)

        if not dry_run:
            zip = zipfile.ZipFile(zip_filename, "w",
                                  compression=zipfile.ZIP_DEFLATED)

            for dirpath, dirnames, filenames in os.walk(base_dir):
                for name in filenames:
                    path = os.path.normpath(os.path.join(dirpath, name))
                    if os.path.isfile(path):
                        zip.write(path, path)
                        if logger is not None:
                            logger.info("adding '%s'", path)
            zip.close()

    return zip_filename

_ARCHIVE_FORMATS = {
    'gztar': (_make_tarball, [('compress', 'gzip')], "gzip'ed tar-file"),
    'bztar': (_make_tarball, [('compress', 'bzip2')], "bzip2'ed tar-file"),
    'tar':   (_make_tarball, [('compress', None)], "uncompressed tar file"),
    'zip':   (_make_zipfile, [],"ZIP file")
    }

def get_archive_formats():
    """Returns a list of supported formats for archiving and unarchiving.

    Each element of the returned sequence is a tuple (name, description)
    """
    formats = [(name, registry[2]) for name, registry in
               _ARCHIVE_FORMATS.items()]
    formats.sort()
    return formats

def register_archive_format(name, function, extra_args=None, description=''):
    """Registers an archive format.

    name is the name of the format. function is the callable that will be
    used to create archives. If provided, extra_args is a sequence of
    (name, value) tuples that will be passed as arguments to the callable.
    description can be provided to describe the format, and will be returned
    by the get_archive_formats() function.
    """
    if extra_args is None:
        extra_args = []
    if not isinstance(function, collections.Callable):
        raise TypeError('The %s object is not callable' % function)
    if not isinstance(extra_args, (tuple, list)):
        raise TypeError('extra_args needs to be a sequence')
    for element in extra_args:
        if not isinstance(element, (tuple, list)) or len(element) !=2 :
            raise TypeError('extra_args elements are : (arg_name, value)')

    _ARCHIVE_FORMATS[name] = (function, extra_args, description)

def unregister_archive_format(name):
    del _ARCHIVE_FORMATS[name]

def make_archive(base_name, format, root_dir=None, base_dir=None, verbose=0,
                 dry_run=0, owner=None, group=None, logger=None):
    """Create an archive file (eg. zip or tar).

    'base_name' is the name of the file to create, minus any format-specific
    extension; 'format' is the archive format: one of "zip", "tar", "bztar"
    or "gztar".

    'root_dir' is a directory that will be the root directory of the
    archive; ie. we typically chdir into 'root_dir' before creating the
    archive.  'base_dir' is the directory where we start archiving from;
    ie. 'base_dir' will be the common prefix of all files and
    directories in the archive.  'root_dir' and 'base_dir' both default
    to the current directory.  Returns the name of the archive file.

    'owner' and 'group' are used when creating a tar archive. By default,
    uses the current owner and group.
    """
    save_cwd = os.getcwd()
    if root_dir is not None:
        if logger is not None:
            logger.debug("changing into '%s'", root_dir)
        base_name = os.path.abspath(base_name)
        if not dry_run:
            os.chdir(root_dir)

    if base_dir is None:
        base_dir = os.curdir

    kwargs = {'dry_run': dry_run, 'logger': logger}

    try:
        format_info = _ARCHIVE_FORMATS[format]
    except KeyError:
        raise ValueError, "unknown archive format '%s'" % format

    func = format_info[0]
    for arg, val in format_info[1]:
        kwargs[arg] = val

    if format != 'zip':
        kwargs['owner'] = owner
        kwargs['group'] = group

    try:
        filename = func(base_name, base_dir, **kwargs)
    finally:
        if root_dir is not None:
            if logger is not None:
                logger.debug("changing back to '%s'", save_cwd)
            os.chdir(save_cwd)

    return filename
